@@ -1,5 +1,6 @@
 import asyncio
 from abc import ABCMeta, abstractmethod
+from threading import Event, Lock, Timer
 from typing import Awaitable, Callable, List, Optional, Set
 
 from loguru import logger
@@ -11,13 +12,21 @@ from tracarbon.conf import tracarbon_configuration as conf
 class Exporter(BaseModel, metaclass=ABCMeta):
     """The Exporter interface."""
 
-    quit: bool = False
+    metrics: List["Metric"]
+    event: Optional[Event] = None
+    stopped: bool = False
 
     class Config:
         """Pydantic configuration."""
 
         arbitrary_types_allowed = True
 
+    def __enter__(self) -> None:
+        self.start()
+
+    def __exit__(self, type, value, traceback) -> None:  # type: ignore
+        self.stop()
+
     @abstractmethod
     async def launch(self, metric: "Metric") -> None:
         """
@@ -28,14 +37,44 @@ class Exporter(BaseModel, metaclass=ABCMeta):
         """
         pass
 
-    async def launch_all(self, metrics: List["Metric"]) -> None:
-        while True:
-            for metric in metrics:
-                logger.debug(f"Running Metric[{metric}].")
-                await self.launch(metric=metric)
-            if self.quit:
-                break
-            await asyncio.sleep(conf.interval_in_seconds)
+    def start(self) -> None:
+        """
+        Start the exporter and a dedicated timer configured with the configured timeout.
+
+        :return:
+        """
+        self.stopped = False
+        if not self.event:
+            self.event = Event()
+
+        def _run() -> None:
+            asyncio.run(self._launch_all())
+            if self.event and not self.stopped and not self.event.is_set():
+                timer = Timer(conf.interval_in_seconds, _run, [])
+                timer.daemon = True
+                timer.start()
+
+        _run()
+
+    def stop(self) -> None:
+        """
+        Stop the explorer and the associated timer.
+
+        :return:
+        """
+        self.stopped = True
+        if self.event:
+            self.event.set()
+
+    async def _launch_all(self) -> None:
+        """
+        Launch the exporter with all the metrics.
+
+        :return:
+        """
+        for metric in self.metrics:
+            logger.debug(f"Running Metric[{metric}].")
+            await self.launch(metric=metric)
 
     @classmethod
     def get_name(cls) -> str:
@@ -49,7 +88,7 @@ class Exporter(BaseModel, metaclass=ABCMeta):
 
 class Metric(BaseModel):
     """
-    Global metric for the exporters.
+    Global metric to use for the exporters.
     """
 
     name: str
